﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

namespace Kivii.Finances
{
    public static class AccountExtension
    {
        private static Dictionary<string, Account> _balanceAccounts = new Dictionary<string, Account>();
        private static Dictionary<string, Account> _bizAccounts = new Dictionary<string, Account>();
        private static Dictionary<string, Account> _discountAccounts = new Dictionary<string, Account>();
        private static Dictionary<string, Account> _personalAccounts = new Dictionary<string, Account>();

        /// <summary>
        /// 获取系统平衡帐户，一个币种只有一个
        /// </summary>
        /// <param name="payment"></param>
        /// <returns></returns>
        public static Account GetBalanceAccount(this CurrencyUnit currency)
        {
            (currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位，不能查询平衡账户");
            var key = KiviiContext.GetUrnKey($"{currency}{KiviiContext.CurrentMember.OrganizationKvid}");
            //var key = KiviiContext.GetUrnKey(currency.ToString());
            //如果包含这个币种，直接返回
            if (_balanceAccounts.ContainsKey(key))
            {
                return _balanceAccounts[key];
            }

            var conn = KiviiContext.GetOpenedDbConnection<Account>();

            //得到所有平衡帐户，并赋值给静态变量，减少数据库操作
            var balanceAccounts = conn.Select<Account>(o => o.Type == AccountType.Balance);
            foreach (var account in balanceAccounts)
            {
                _balanceAccounts[KiviiContext.GetUrnKey($"{currency}{KiviiContext.CurrentMember.OrganizationKvid}")] = account;
            }
            //如果从数据库中得到了，直接返回
            if (_balanceAccounts.ContainsKey(key))
            {
                return _balanceAccounts[key];
            }

            ////每个币种只有一个平衡帐户，通过自定义的Kvid防止多系统或多线程冲突引起在数据库中创建多个平衡帐户，Kvid的开头为BALANCE的象形写法
            //var currencyString = ((int)currency).ToString().PadLeft(4, '0');
            var balanceKvid = Guid.NewGuid();//new Guid($"BA1A0CE0-{currencyString}-0000-0000-000000000000");
            Account rtns = new Account();
            rtns.Kvid = balanceKvid;
            rtns.Type = AccountType.Balance;
            rtns.Currency = currency;
            rtns.OwnerKvid = KiviiContext.CurrentMember.OrganizationKvid;
            rtns.OwnerName = KiviiContext.CurrentMember.OrganizationName;
            rtns.Name = "系统平衡帐户";
            rtns.SerialNumber = "";
            rtns.CreditLine = 0;
            rtns.Amount = 0;
            rtns.Summary = "系统内部平衡帐户，禁止操作";
            rtns.Remark = "";
            if (conn.Insert(rtns) == 1)
            {
                _balanceAccounts[KiviiContext.GetUrnKey($"{rtns.Currency}{KiviiContext.CurrentMember.OrganizationKvid}")] = rtns;
                return rtns;
            }
            throw new Exception("系统创建平衡帐户失败，请联系系统管理员。");
        }

        /// <summary>
        /// 获取折扣账户
        /// </summary>
        /// <param name="currency"></param>
        /// <returns></returns>
        public static Account GetDiscountAccount(this CurrencyUnit currency, Guid? ownerKvid = null, string ownerName = null)
        {
            if (ownerKvid == null)
            {
                ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                ownerName = KiviiContext.CurrentMember.DepartmentName;
            }
            if(ownerKvid==Guid.Empty)
            {
                ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                ownerName = KiviiContext.CurrentMember.DepartmentName;
            }
            (currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位，不能查询折扣账户");
            var key = KiviiContext.GetUrnKey($"{AccountType.Discount}{currency}{ownerKvid}");
            //如果包含这个币种，直接返回
            if (_discountAccounts.ContainsKey(key))
            {
                return _discountAccounts[key];
            }

            var conn = KiviiContext.GetOpenedDbConnection<Account>();

            //得到所有平衡帐户，并赋值给静态变量，减少数据库操作
            var bizAccounts = conn.Select<Account>(o => o.Type == AccountType.Discount);
            foreach (var account in bizAccounts)
            {
                _discountAccounts[KiviiContext.GetUrnKey($"{AccountType.Discount}{account.Currency}{ownerKvid}")] = account;
            }
            //如果从数据库中得到了，直接返回
            if (_discountAccounts.ContainsKey(key))
            {
                return _discountAccounts[key];
            }

            var bizKvid = Guid.NewGuid();
            Account rtns = new Account();
            rtns.Kvid = bizKvid;
            rtns.Type = AccountType.Discount;
            rtns.Currency = currency;
            rtns.OwnerKvid = ownerKvid.Value;
            rtns.OwnerName = ownerName;
            rtns.Name = $"折扣帐户({rtns.OwnerName})";
            rtns.SerialNumber = "";
            rtns.CreditLine = 10000000000;
            rtns.Amount = 0;
            rtns.Summary = "";
            rtns.Remark = "";
            if (conn.Insert(rtns) == 1)
            {
                _discountAccounts[KiviiContext.GetUrnKey($"{AccountType.Discount}{rtns.Currency}{ownerKvid}")] = rtns;
                return rtns;
            }
            throw new Exception("系统创建折扣帐户失败，请联系系统管理员。");
        }

        /// <summary>
        /// 获取业务账户
        /// </summary>
        /// <param name="currency"></param>
        /// <returns></returns>
        public static Account GetBizAccount(this CurrencyUnit currency,Guid? ownerKvid = null, string ownerName = null)
        {
            (currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位，不能查询业务账户");
            if (ownerKvid == null)
            {
                ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                ownerName = KiviiContext.CurrentMember.DepartmentName;
            }
            if (ownerKvid == Guid.Empty)
            {
                ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                ownerName = KiviiContext.CurrentMember.DepartmentName;
            }
            var key = KiviiContext.GetUrnKey($"{AccountType.Biz}{currency}{ownerKvid}");
            //如果包含这个币种，直接返回
            if (_bizAccounts.ContainsKey(key))
            {
                return _bizAccounts[key];
            }

            var conn = KiviiContext.GetOpenedDbConnection<Account>();

            //得到所有平衡帐户，并赋值给静态变量，减少数据库操作
            var bizAccounts = conn.Select<Account>(o => o.Type == AccountType.Biz);
            foreach (var account in bizAccounts)
            {
                _bizAccounts[KiviiContext.GetUrnKey($"{AccountType.Biz}{account.Currency}{ownerKvid}")] = account;
            }
            //如果从数据库中得到了，直接返回
            if (_bizAccounts.ContainsKey(key))
            {
                return _bizAccounts[key];
            }

            var bizKvid = Guid.NewGuid();
            Account rtns = new Account();
            rtns.Kvid = bizKvid;
            rtns.Type = AccountType.Biz;
            rtns.Currency = currency;
            rtns.OwnerKvid = ownerKvid.Value;
            rtns.OwnerName = ownerName;
            rtns.Name = $"业务帐户({rtns.OwnerName})";
            rtns.SerialNumber = "";
            rtns.CreditLine = 0;
            rtns.Amount = 0;
            rtns.Summary = "";
            rtns.Remark = "";
            if (conn.Insert(rtns) == 1)
            {
                _bizAccounts[KiviiContext.GetUrnKey($"{AccountType.Biz}{rtns.Currency}{ownerKvid}")] = rtns;
                return rtns;
            }
            throw new Exception("系统创建业务帐户失败，请联系系统管理员。");
        }

        /// <summary>
        /// 获取个人帐户,例如现金账户
        /// </summary>
        /// <param name="payment"></param>
        /// <returns></returns>
        public static Account GetPersonalAccount(this CurrencyUnit currency, AccountType accountType, Guid? ownerKvid = null, string ownerName = null)
        {
            (currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位，不能查询个人账户");
            (accountType != AccountType.Cash && accountType != AccountType.Pos).ThrowIfTrue("不支持的账户类型!,个人账户仅支持现金和刷卡账户!");
            if (ownerKvid == null)
            {
                ownerKvid = KiviiContext.CurrentMember.Kvid;
                ownerName = KiviiContext.CurrentMember.FullName;
            }
            if (ownerKvid == Guid.Empty)
            {
                ownerKvid = KiviiContext.CurrentMember.Kvid;
                ownerName = KiviiContext.CurrentMember.FullName;
            }
            var key = KiviiContext.GetUrnKey($"{accountType}{currency}{ownerKvid}");
            //如果包含这个币种，直接返回
            if (_personalAccounts.ContainsKey(key))
            {
                return _personalAccounts[key];
            }

            var conn = KiviiContext.GetOpenedDbConnection<Account>();

            //得到所有个人帐户，并赋值给静态变量，减少数据库操作
            var personalAccount = conn.Single<Account>(o => o.Type == accountType && o.OwnerKvid == ownerKvid);
            if (personalAccount != null) _personalAccounts[KiviiContext.GetUrnKey($"{personalAccount.Type}{currency}{personalAccount.OwnerKvid}")] = personalAccount;
            //如果从数据库中得到了，直接返回
            if (_personalAccounts.ContainsKey(key))
            {
                return _personalAccounts[key];
            }

            Account rtns = new Account();
            rtns.Kvid = Guid.NewGuid();
            rtns.Type = accountType;
            rtns.Currency = currency;
            rtns.OwnerKvid = ownerKvid.Value;
            rtns.OwnerName = ownerName;
            rtns.Name = $"{rtns.OwnerName}的{accountType}账户";
            rtns.SerialNumber = $"{accountType}{(DateTime.Now.Ticks - DateTime.Parse("1970-01-01").Ticks) / 10000}";
            rtns.CreditLine = 0;
            rtns.Amount = 0;
            rtns.Summary = "";
            rtns.Remark = "";
            if (conn.Insert(rtns) == 1)
            {
                _personalAccounts[KiviiContext.GetUrnKey($"{rtns.Type}{currency}{rtns.OwnerKvid}")] = rtns;
                return rtns;
            }
            throw new Exception("系统创建个人帐户失败，请联系系统管理员。");
        }

        /// <summary>
        /// complate!
        /// 新增账户明细
        /// </summary>
        /// <param name="accountDetail"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Account Insert(this AccountDetail accountDetail, IDbConnection conn = null)
        {
            accountDetail.ThrowIfNull("账户明细不能为空!");
            (accountDetail.Amount == 0).ThrowIfFalse("账户余额不能被赋除0之外的值!");
            (accountDetail.AmountPayment <= 0).ThrowIfTrue("账户明细发生金额不能少于0!");
            accountDetail.AccountKvid.ThrowIfEmpty("账户Kvid不能为空!");
            //accountDetail.PayeeAccountKvid.ThrowIfEmpty("收款账户不能为空!");
            //accountDetail.PayerAccountKvid.ThrowIfEmpty("付款账户不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<AccountDetail>();

            var account = conn.SingleById<Account>(accountDetail.AccountKvid);
            account.ThrowIfNull("未找到此账户!");

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                decimal amount = 0;
                if (accountDetail.PayerAccountKvid == account.Kvid) amount = 0 - accountDetail.AmountPayment;
                if (accountDetail.PayeeAccountKvid == account.Kvid) amount = accountDetail.AmountPayment;
                //updateAdd中Amount是累加的 
                (conn.UpdateAdd(() => new Account { Amount = amount }, o => o.Kvid == account.Kvid) == 1).ThrowIfFalse("更新帐户数据错误");
                accountDetail.Amount = account.Amount + amount;
                (conn.Insert(accountDetail) == 1).ThrowIfFalse("插入帐户明细错误！");
                trans?.Commit();
                return account;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
}
