﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "非业务处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentUnBiz : RestfulExecution<Payment>
    {
        public decimal Amount { get; set; }

        public Guid Kvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Amount <= 0) throw new Exception("金额不能为小于0!");
            if (Kvid == Guid.Empty) throw new Exception("请选择要非业务收款处理的到账!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payment = conn.SingleById<Payment>(Kvid);
            if (payment == null) throw new Exception("收款不存在!");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以操作!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可操作!");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可操作!");
            if (payment.AmountSplited + Amount > payment.Amount) throw new Exception("操作金额超出范围!");
            //if (payment.AmountInvoice > 0) throw new Exception("已开票,无法非业务处理!");
            if (payment.AmountUsed > 0) throw new Exception("已使用金额,无法非业务处理!");

            var split = payment.UnBizSplit(Amount, PaymentType.UnBiz);

            #region 记录日志
            var log = new EntityLog<Payment>();
            log.OwnerKvid = payment.Kvid;
            log.BizId = split.SerialNumber;
            log.BizKvid = split.Kvid;
            log.BizType = typeof(Payment).FullName;
            log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行非业务收款操作,金额为:{split.Amount}元.";
            #endregion

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                conn.Insert(split);
                payment.RecalculateAmountSplit(conn);
                conn.Insert(log);

                rtns.Results.Add(split);

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "批量非业务处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentMultiUnBiz : RestfulExecution<Payment>
    {
        public List<Guid> Kvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要非业务收款处理的到账!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.Bank)) throw new Exception("非银行到账不可以操作!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可操作!");
            if (payments.Exists(o => o.AmountSplited >= o.Amount)) throw new Exception("无剩余收款可操作!");
            //if (payments.Exists(o => o.AmountInvoice > 0)) throw new Exception("已开票,无法非业务处理!");
            if (payments.Exists(o => o.AmountUsed > 0)) throw new Exception("已使用,无法非业务处理!");

            var banks = new List<Payment>();
            var splits = new List<Payment>();
            var logs = new List<EntityLog<Payment>>();

            foreach (var payment in payments)
            {
                var amount = payment.Amount - payment.AmountSplited;
                if (amount <= 0) continue;
                var split = payment.UnBizSplit(amount, PaymentType.UnBiz);
                banks.Add(payment);
                splits.Add(split);

                #region 记录日志
                var log = new EntityLog<Payment>();
                log.OwnerKvid = payment.Kvid;
                log.BizId = split.SerialNumber;
                log.BizKvid = split.Kvid;
                log.BizType = typeof(Payment).FullName;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行非业务收款操作,金额为:{split.Amount}元.";
                logs.Add(log);
                #endregion
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            //开启事务进行数据库操作
            var trans = conn.OpenTransaction();
            try
            {
                foreach (var split in splits)
                {
                    conn.Insert(split);
                    rtns.Results.Add(split);
                }
                foreach (var item in banks)
                {
                    item.RecalculateAmountSplit(conn);
                }
                foreach (var log in logs)
                {
                    conn.Insert(log);
                }

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "撤销非业务处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentUnBizCancel : RestfulExecution<Payment>
    {
        public List<Guid> Kvids { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要撤销非业务的收款!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.UnBiz)) throw new Exception("非银行到账不可以操作!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可撤销操作!");
            if (payments.Exists(o => o.AmountUsed > 0)) throw new Exception("存在到账已使用,不可撤销操作!");
            (conn.Exists<InvoiceApply>(o => o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty && o.AmountInvoice != o.Amount && Sql.In(o.BizKvid, payments.ConvertAll(p => p.Kvid)))).ThrowIfTrue("存在申请开票中的到账,暂无法撤销认领!");

            var parentKvids = payments.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var parents = conn.SelectByIds<Payment>(parentKvids);

            var parentUpdates = new List<Payment>();
            var splitOffsets = new List<Payment>();
            var offsets = new List<Payment>();

            foreach (var split in payments)
            {
                var parent = parents.FirstOrDefault(o => o.Kvid == split.ParentKvid);
                if (parent == null) continue;

                var offset = split.Offset($"非业务撤销:{Remark}");
                offsets.Add(offset);
                splitOffsets.Add(split);
                parentUpdates.Add(parent);
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                foreach (var item in splitOffsets)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }
                foreach (var item in offsets)
                {
                    conn.Insert(item);
                }
                foreach (var item in parentUpdates)
                {
                    item.RecalculateAmountSplit(conn);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "退款处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentRefund : RestfulExecution<Payment>
    {
        public decimal Amount { get; set; }

        public Guid Kvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Amount <= 0) throw new Exception("金额不能为小于0!");
            if (Kvid == Guid.Empty) throw new Exception("请选择要退款处理的到账!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payment = conn.SingleById<Payment>(Kvid);
            if (payment == null) throw new Exception("收款不存在!");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以操作!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可操作!");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可操作!");
            if (payment.AmountSplited + Amount > payment.Amount) throw new Exception("操作金额超出范围!");
            //if (payment.AmountInvoice > 0) throw new Exception("已开票,无法退款处理!");
            if (payment.AmountUsed > 0) throw new Exception("已使用金额,无法退款处理!");

            var split = payment.UnBizSplit(Amount, PaymentType.Refund);

            #region 记录日志
            var log = new EntityLog<Payment>();
            log.OwnerKvid = payment.Kvid;
            log.BizId = split.SerialNumber;
            log.BizKvid = split.Kvid;
            log.BizType = typeof(Payment).FullName;
            log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行退款操作,金额为:{split.Amount}元.";
            #endregion

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                conn.Insert(split);
                payment.RecalculateAmountSplit(conn);
                conn.Insert(log);

                rtns.Results.Add(split);

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "批量退款处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentMultiRefund : RestfulExecution<Payment>
    {
        public List<Guid> Kvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要退款处理的到账!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.Bank)) throw new Exception("非银行到账不可以操作!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可操作!");
            if (payments.Exists(o => o.AmountSplited >= o.Amount)) throw new Exception("无剩余收款可操作!");
            //if (payments.Exists(o => o.AmountInvoice > 0)) throw new Exception("已开票,无法退款处理!");
            if (payments.Exists(o => o.AmountUsed > 0)) throw new Exception("已使用,无法退款处理!");

            var banks = new List<Payment>();
            var splits = new List<Payment>();
            var logs = new List<EntityLog<Payment>>();

            foreach (var payment in payments)
            {
                var amount = payment.Amount - payment.AmountSplited;
                if (amount <= 0) continue;
                var split = payment.UnBizSplit(amount, PaymentType.Refund);
                banks.Add(payment);
                splits.Add(split);

                #region 记录日志
                var log = new EntityLog<Payment>();
                log.OwnerKvid = payment.Kvid;
                log.BizId = split.SerialNumber;
                log.BizKvid = split.Kvid;
                log.BizType = typeof(Payment).FullName;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = KiviiContext.CurrentMember.FullName + $"在  {log.Title} 对流水号为:{payment.SerialNumber}的收款进行退款操作,金额为:{split.Amount}元.";
                logs.Add(log);
                #endregion
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            //开启事务进行数据库操作
            var trans = conn.OpenTransaction();
            try
            {
                foreach (var split in splits)
                {
                    conn.Insert(split);
                    rtns.Results.Add(split);
                }
                foreach (var item in banks)
                {
                    item.RecalculateAmountSplit(conn);
                }
                foreach (var log in logs)
                {
                    conn.Insert(log);
                }

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "撤销退款处理")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentRefundCancel : RestfulExecution<Payment>
    {
        public List<Guid> Kvids { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("请选择要撤销退款的收款!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            if (payments == null) throw new Exception("收款不存在!");
            if (payments.Exists(o => o.Type != PaymentType.Refund)) throw new Exception("非银行到账不可以操作!");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("已冲销收款不可撤销操作!");
            if (payments.Exists(o => o.AmountUsed > 0)) throw new Exception("存在到账已使用,不可撤销操作!");
            (conn.Exists<InvoiceApply>(o => o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty && o.AmountInvoice != o.Amount && Sql.In(o.BizKvid, payments.ConvertAll(p => p.Kvid)))).ThrowIfTrue("存在申请开票中的到账,暂无法撤销认领!");

            var parentKvids = payments.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var parents = conn.SelectByIds<Payment>(parentKvids);

            var parentUpdates = new List<Payment>();
            var splitOffsets = new List<Payment>();
            var offsets = new List<Payment>();

            foreach (var split in payments)
            {
                var parent = parents.FirstOrDefault(o => o.Kvid == split.ParentKvid);
                if (parent == null) continue;

                var offset = split.Offset($"退款撤销:{Remark}");
                offsets.Add(offset);
                splitOffsets.Add(split);
                parentUpdates.Add(parent);
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                foreach (var item in splitOffsets)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }
                foreach (var item in offsets)
                {
                    conn.Insert(item);
                }
                foreach (var item in parentUpdates)
                {
                    item.RecalculateAmountSplit(conn);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
